//-----------------------------------------------------------------------
//  Java Indexed Serialization Package
//  ----------------------------------
//  VFlowLayout.java
//-----------------------------------------------------------------------
//
//  COPYRIGHT NOTICE, DISCLAIMER, and LICENSE:
//  
//  If you modify this file, you may insert additional notices
//  immediately following this sentence.
//  
//  Copyright 1996-2002 Scott Robert Ladd.
//  All rights reserved, except as noted herein.
//  
//  This computer program source file is supplied "AS IS". Scott Robert
//  Ladd (hereinafter referred to as "Author") disclaims all warranties,
//  expressed or implied, including, without limitation, the warranties
//  of merchantability and of fitness for any purpose. The Author
//  assumes no liability for direct, indirect, incidental, special,
//  exemplary, or consequential damages, which may result from the use
//  of this software, even if advised of the possibility of such damage.
//  
//  The Author hereby grants anyone permission to use, copy, modify, and
//  distribute this source code, or portions hereof, for any purpose,
//  without fee, subject to the following restrictions:
//  
//      1. The origin of this source code must not be misrepresented.
//  
//      2. Altered versions must be plainly marked as such and must not
//         be misrepresented as being the original source.
//  
//      3. This Copyright notice may not be removed or altered from any
//         source or altered source distribution.
//  
//  The Author specifically permits (without fee) and encourages the use
//  of this source code for entertainment, education, or decoration. If
//  you use this source code in a product, acknowledgment is not required
//  but would be appreciated.
//  
//  Acknowledgement:
//      This license is based on the wonderful simple license that
//      accompanies libpng.
//
//-----------------------------------------------------------------------
//  For more information on this software package, please visit
//  Scott's web site, Coyote Gulch Productions, at:
//
//      http://www.coyotegulch.com
//-----------------------------------------------------------------------

import java.awt.*;

public class VFlowLayout implements LayoutManager 
{
    // top-bottom alignment
    public static final int TOP    =  0;
    public static final int MIDDLE =  1;
    public static final int BOTTOM =  2;
    
    // left-right alignment
    public static final int LEFT   =  0;
    public static final int CENTER =  1;
    public static final int RIGHT  =  2;

    // fields
    private int m_anchor;
    private int m_align;
    private int m_hgap;
    private int m_vgap;

    // exceptions
    private static final IllegalArgumentException err1 = new IllegalArgumentException("invalid VFlowLayout alignment");
    private static final IllegalArgumentException err2 = new IllegalArgumentException("invalid VFlowLayout hgap");
    private static final IllegalArgumentException err3 = new IllegalArgumentException("invalid VFlowLayout vgap");

    // constructors
    public VFlowLayout()
    {
        this(TOP,LEFT,5,5);
    }

    public VFlowLayout(int anchor, int alignment)
    {
        this(anchor,alignment,5,5);
    }

    public VFlowLayout(int anchor, int alignment, int hgap, int vgap)
    {
        // verify parameters
        if ((anchor < TOP) || (anchor > BOTTOM) || (alignment < LEFT) || (alignment > RIGHT))
            throw err1;

        if (hgap < 0)
            throw err2;

        if (vgap < 0)
            throw err3;

        // store parameters
        m_anchor = anchor;
        m_align  = alignment;
        m_hgap   = hgap;
        m_vgap   = vgap;
    }

    public int getAnchor()
    {
        return m_anchor;
    }

    public int getAlignment()
    {
        return m_align;
    }

    public void setHgap(int hgap)
    {
        // verify parameters
        if (hgap < 0)
            throw err2;

        // store parameters
        m_hgap  = hgap;
    }

    public int getHgap()
    {
        return m_hgap;
    }

    public void setVgap(int vgap)
    {
        // verify parameters
        if (vgap < 0)
            throw err3;

        // store parameters
        m_vgap  = vgap;
    }

    public int getVgap()
    {
        return m_vgap;
    }

    // methods
    public void addLayoutComponent(String name, Component comp)
    {
        // does nothing in this class
    }

    public void removeLayoutComponent(Component comp)
    {
        // does nothing in this class
    }

    public Dimension preferredLayoutSize(Container target)
    {
        // create a zero dimension
        Dimension dim = new Dimension(0,0);

        // how many components in container?
        int n = target.getComponentCount();

        // compute width and height
        for (int i = 0; i < n; ++i)
        {
            // reference target component
            Component m = target.getComponent(i);

            // only compute for visible components
            if (m.isVisible())
            {
                Dimension d = m.getPreferredSize();

                dim.width   = Math.max(dim.width,d.width);
                dim.height += d.height + m_vgap;
            }
        }

        // include insets in dimensions
        Insets ins = target.getInsets();

        dim.width  += ins.left + ins.right + 2 * m_hgap;
        dim.height += ins.top + ins.bottom + m_vgap;

        // return results
        return dim;
    }

    public Dimension minimumLayoutSize(Container target)
    {
        // create a zero dimension
        Dimension dim = new Dimension(0,0);

        // how many components in container?
        int n = target.getComponentCount();

        // compute width and height
        for (int i = 0; i < n; ++i)
        {
            // reference target component
            Component m = target.getComponent(i);

            // only compute for visible components
            if (m.isVisible())
            {
                Dimension d = m.getMinimumSize();

                dim.width   = Math.max(dim.width,d.width);
                dim.height += d.height + m_vgap;
            }
        }

        // include insets in dimensions
        Insets ins = target.getInsets();

        dim.width  += ins.left + ins.right + 2 * m_hgap;
        dim.height += ins.top + ins.bottom + m_vgap;

        // return results
        return dim;
    }

    private void moveComponents(Container target, int x, int y, int width, int height, int colStart, int colEnd)
    {
        switch (m_anchor)
        {
        case TOP:
            break;
        case MIDDLE:
            y += (height / 2);
            break;
        case BOTTOM:
            y += height;
            break;
        }

        for (int i = colStart; i < colEnd; ++i)
        {
            Component m = target.getComponent(i);
            Dimension d = m.getSize();
            int xpos = x;
            
            switch (m_align)
            {
                case LEFT:
                    break;
                case CENTER:
                    xpos += (width - d.width) / 2;
                    break;
                case RIGHT:
                    xpos += (width - d.width);
                    break;
            }

            if (m.isVisible())
            {
                m.setLocation(xpos, y);
                y += m_vgap + d.height;
            }
        }
    }

    public void layoutContainer(Container target)
    {
        Insets ins = target.getInsets();
        int maxH = target.getSize().height - (ins.top + ins.bottom + m_vgap * 2);
        int n = target.getComponentCount();
        int x = ins.left + m_vgap;
        int y = 0;
        int c = 0;
        int start = 0;

        for (int i = 0; i < n; ++i)
        {
            Component m = target.getComponent(i);

            if (m.isVisible())
            {
                Dimension d = m.getPreferredSize();
                m.setSize(d.width,d.height);

                if ((y == 0) || ((y + d.height) <= maxH))
                {
                    if (y > 0)
                        y += m_vgap;

                    y += d.height;

                    c = Math.max(c,d.width);
                }
                else
                {
                    moveComponents(target, x, ins.top + m_vgap, c, maxH - y, start, i);
                    y  = d.height;
                    x += m_hgap + c;
                    c  = d.width;
                    start = i;
                }
            }
        }
        
        x += (target.getSize().width - c) / 2 - m_hgap;
        
        moveComponents(target, x, ins.top + m_vgap, c, maxH - y, start, n);
    }

    public String toString()
    {
        String str;

        switch (m_anchor)
        {
        case TOP:    str = "top";    break;
        case BOTTOM: str = "bottom"; break;
        case MIDDLE: str = "middle"; break;
        default:     str = "invalid";
        }

        switch (m_align)
        {
        case LEFT:   str += "left";   break;
        case CENTER: str += "center"; break;
        case RIGHT:  str += "right";  break;
        default:     str += "invalid";
        }

        return getClass().getName() + "[hgap=" + m_hgap + ",vgap=" + m_vgap + ",align=" + str + "]";
    }
}
